@@ -43,6 +43,7 @@ import (
 	"github.com/gohugoio/hugo/hugofs"
 	"github.com/gohugoio/hugo/hugolib"
 	"github.com/gohugoio/hugo/hugolib/filesystems"
+	"github.com/gohugoio/hugo/identity"
 	"github.com/gohugoio/hugo/livereload"
 	"github.com/gohugoio/hugo/resources/page"
 	"github.com/gohugoio/hugo/watcher"
@@ -343,6 +344,27 @@ func (c *hugoBuilder) newWatcher(pollIntervalStr string, dirList ...string) (*wa
 	go func() {
 		for {
 			select {
+			case changes := <-c.r.changesFromBuild:
+				c.errState.setBuildErr(nil)
+				unlock, err := h.LockBuild()
+				if err != nil {
+					c.r.logger.Errorln("Failed to acquire a build lock: %s", err)
+					return
+				}
+				c.changeDetector.PrepareNew()
+				err = c.rebuildSitesForChanges(changes)
+				if err != nil {
+					c.r.logger.Errorln("Error while watching:", err)
+				}
+				if c.s != nil && c.s.doLiveReload {
+					doReload := c.changeDetector == nil || len(c.changeDetector.changed()) > 0
+					doReload = doReload || c.showErrorInBrowser && c.errCount() > 0
+					if doReload {
+						livereload.ForceRefresh()
+					}
+				}
+				unlock()
+
 			case evs := <-watcher.Events:
 				unlock, err := h.LockBuild()
 				if err != nil {
@@ -722,6 +744,20 @@ func (c *hugoBuilder) handleEvents(watcher *watcher.Batcher,
 	staticEvents := []fsnotify.Event{}
 	dynamicEvents := []fsnotify.Event{}
 
+	filterDuplicateEvents := func(evs []fsnotify.Event) []fsnotify.Event {
+		seen := make(map[string]bool)
+		var n int
+		for _, ev := range evs {
+			if seen[ev.Name] {
+				continue
+			}
+			seen[ev.Name] = true
+			evs[n] = ev
+			n++
+		}
+		return evs[:n]
+	}
+
 	h, err := c.hugo()
 	if err != nil {
 		c.r.logger.Errorln("Error getting the Hugo object:", err)
@@ -743,6 +779,7 @@ func (c *hugoBuilder) handleEvents(watcher *watcher.Batcher,
 		istemp := strings.HasSuffix(ext, "~") ||
 			(ext == ".swp") || // vim
 			(ext == ".swx") || // vim
+			(ext == ".bck") || // helix
 			(ext == ".tmp") || // generic temp file
 			(ext == ".DS_Store") || // OSX Thumbnail
 			baseName == "4913" || // vim
@@ -811,6 +848,11 @@ func (c *hugoBuilder) handleEvents(watcher *watcher.Batcher,
 		}
 	}
 
+	lrl := c.r.logger.InfoCommand("livereload")
+
+	staticEvents = filterDuplicateEvents(staticEvents)
+	dynamicEvents = filterDuplicateEvents(dynamicEvents)
+
 	if len(staticEvents) > 0 {
 		c.printChangeDetected("Static files")
 
@@ -831,19 +873,20 @@ func (c *hugoBuilder) handleEvents(watcher *watcher.Batcher,
 		if c.s != nil && c.s.doLiveReload {
 			// Will block forever trying to write to a channel that nobody is reading if livereload isn't initialized
 
-			// force refresh when more than one file
 			if !c.errState.wasErr() && len(staticEvents) == 1 {
-				ev := staticEvents[0]
 				h, err := c.hugo()
 				if err != nil {
 					c.r.logger.Errorln("Error getting the Hugo object:", err)
 					return
 				}
-				path := h.BaseFs.SourceFilesystems.MakeStaticPathRelative(ev.Name)
+
+				path := h.BaseFs.SourceFilesystems.MakeStaticPathRelative(staticEvents[0].Name)
 				path = h.RelURL(paths.ToSlashTrimLeading(path), false)
 
+				lrl.Logf("refreshing static file %q", path)
 				livereload.RefreshPath(path)
 			} else {
+				lrl.Logf("got %d static file change events, force refresh", len(staticEvents))
 				livereload.ForceRefresh()
 			}
 		}
@@ -867,20 +910,30 @@ func (c *hugoBuilder) handleEvents(watcher *watcher.Batcher,
 		}()
 
 		if c.s != nil && c.s.doLiveReload {
-			if len(partitionedEvents.ContentEvents) == 0 && len(partitionedEvents.AssetEvents) > 0 {
-				if c.errState.wasErr() {
-					livereload.ForceRefresh()
-					return
-				}
-				changed := c.changeDetector.changed()
-				if c.changeDetector != nil && len(changed) == 0 {
+			if c.errState.wasErr() {
+				livereload.ForceRefresh()
+				return
+			}
+
+			changed := c.changeDetector.changed()
+			if c.changeDetector != nil {
+				lrl.Logf("build changed %d files", len(changed))
+				if len(changed) == 0 {
 					// Nothing has changed.
 					return
-				} else if len(changed) == 1 {
-					pathToRefresh := h.PathSpec.RelURL(paths.ToSlashTrimLeading(changed[0]), false)
-					livereload.RefreshPath(pathToRefresh)
+				}
+			}
+
+			// If this change set also contains one or more CSS files, we need to
+			// refresh these as well.
+			var cssChanges []string
+			var otherChanges []string
+
+			for _, ev := range changed {
+				if strings.HasSuffix(ev, ".css") {
+					cssChanges = append(cssChanges, ev)
 				} else {
-					livereload.ForceRefresh()
+					otherChanges = append(otherChanges, ev)
 				}
 			}
 
@@ -896,11 +949,36 @@ func (c *hugoBuilder) handleEvents(watcher *watcher.Batcher,
 					}
 				}
 
-				if p != nil {
-					livereload.NavigateToPathForPort(p.RelPermalink(), p.Site().ServerPort())
+				if p != nil && p.RelPermalink() != "" {
+					link, port := p.RelPermalink(), p.Site().ServerPort()
+					lrl.Logf("navigating to %q using port %d", link, port)
+					livereload.NavigateToPathForPort(link, port)
 				} else {
+					lrl.Logf("no page to navigate to, force refresh")
 					livereload.ForceRefresh()
 				}
+			} else if len(otherChanges) > 0 {
+				if len(otherChanges) == 1 {
+					// Allow single changes to be refreshed without a full page reload.
+					pathToRefresh := h.PathSpec.RelURL(paths.ToSlashTrimLeading(otherChanges[0]), false)
+					lrl.Logf("refreshing %q", pathToRefresh)
+					livereload.RefreshPath(pathToRefresh)
+				} else if len(cssChanges) == 0 {
+					lrl.Logf("force refresh")
+					livereload.ForceRefresh()
+				}
+			}
+
+			if len(cssChanges) > 0 {
+				// Allow some time for the live reload script to get reconnected.
+				if len(otherChanges) > 0 {
+					time.Sleep(200 * time.Millisecond)
+				}
+				for _, ev := range cssChanges {
+					pathToRefresh := h.PathSpec.RelURL(paths.ToSlashTrimLeading(ev), false)
+					lrl.Logf("refreshing CSS %q", pathToRefresh)
+					livereload.RefreshPath(pathToRefresh)
+				}
 			}
 		}
 	}
@@ -969,7 +1047,7 @@ func (c *hugoBuilder) loadConfig(cd *simplecobra.Commandeer, running bool) error
 		"fastRenderMode": c.fastRenderMode,
 	})
 
-	conf, err := c.r.ConfigFromProvider(c.r.configVersionID.Load(), flagsToCfg(cd, cfg))
+	conf, err := c.r.ConfigFromProvider(configKey{counter: c.r.configVersionID.Load()}, flagsToCfg(cd, cfg))
 	if err != nil {
 		return err
 	}
@@ -1019,13 +1097,26 @@ func (c *hugoBuilder) rebuildSites(events []fsnotify.Event) error {
 	return h.Build(hugolib.BuildCfg{NoBuildLock: true, RecentlyVisited: c.visitedURLs, ErrRecovery: c.errState.wasErr()}, events...)
 }
 
+func (c *hugoBuilder) rebuildSitesForChanges(ids []identity.Identity) error {
+	c.errState.setBuildErr(nil)
+	h, err := c.hugo()
+	if err != nil {
+		return err
+	}
+	whatChanged := &hugolib.WhatChanged{}
+	whatChanged.Add(ids...)
+	err = h.Build(hugolib.BuildCfg{NoBuildLock: true, WhatChanged: whatChanged, RecentlyVisited: c.visitedURLs, ErrRecovery: c.errState.wasErr()})
+	c.errState.setBuildErr(err)
+	return err
+}
+
 func (c *hugoBuilder) reloadConfig() error {
 	c.r.Reset()
 	c.r.configVersionID.Add(1)
 
 	if err := c.withConfE(func(conf *commonConfig) error {
 		oldConf := conf
-		newConf, err := c.r.ConfigFromConfig(c.r.configVersionID.Load(), conf)
+		newConf, err := c.r.ConfigFromConfig(configKey{counter: c.r.configVersionID.Load()}, conf)
 		if err != nil {
 			return err
 		}
